# -*- coding: utf-8 -*-
"""DNACenterAPI tag API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_create_tag(obj):
    json_schema_validate('jsd_1399891c42a8be64_v1_2_10').validate(obj)
    return True


def create_tag(api):
    endpoint_result = api.tag.create_tag(
        description=None,
        dynamicRules=None,
        id=None,
        instanceTenantId=None,
        name='InterestingTool01',
        systemTag=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_create_tag(api):
    assert is_valid_create_tag(
        create_tag(api)
    )


def is_valid_get_tag(obj):
    json_schema_validate('jsd_ee9aab01487a8896_v1_2_10').validate(obj)
    return True


def get_tag(api):
    endpoint_result = api.tag.get_tag(
        additional_info_attributes=None,
        additional_info_name_space=None,
        field=None,
        level=None,
        limit=None,
        name=None,
        offset=None,
        order='des',
        size=None,
        sort_by='name',
        system_tag=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag(api):
    assert is_valid_get_tag(
        get_tag(api)
    )


def is_valid_get_tag_created(obj):
    json_schema_validate('jsd_ee9aab01487a8896_v1_2_10').validate(obj)
    return True


def get_tag_created(api):
    endpoint_result = api.tag.get_tag(
        additional_info_attributes=None,
        additional_info_name_space=None,
        field=None,
        level=None,
        limit=None,
        name='InterestingTool01',
        offset=None,
        order=None,
        size=None,
        sort_by=None,
        system_tag=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_created(api):
    assert is_valid_get_tag_created(
        get_tag_created(api)
    )


def is_valid_get_tag_by_id(obj):
    json_schema_validate('jsd_c1a359b14c89b573_v1_2_10').validate(obj)
    return True


def get_tag_by_id(api):
    endpoint_result = api.tag.get_tag_by_id(
        id=get_tag(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_by_id(api):
    assert is_valid_get_tag_by_id(
        get_tag_by_id(api)
    )


def is_valid_get_tag_by_id_created(obj):
    json_schema_validate('jsd_c1a359b14c89b573_v1_2_10').validate(obj)
    return True


def get_tag_by_id_created(api):
    endpoint_result = api.tag.get_tag_by_id(
        id=get_tag_created(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_by_id_created(api):
    assert is_valid_get_tag_by_id_created(
        get_tag_by_id_created(api)
    )


def is_valid_get_tag_count(obj):
    json_schema_validate('jsd_8091a9b84bfba53b_v1_2_10').validate(obj)
    return True


def get_tag_count(api):
    endpoint_result = api.tag.get_tag_count(
        attribute_name=None,
        level=None,
        name=None,
        name_space=None,
        size=None,
        system_tag=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_count(api):
    assert is_valid_get_tag_count(
        get_tag_count(api)
    )


def is_valid_get_tag_resource_types(obj):
    json_schema_validate('jsd_4695090d403b8eaa_v1_2_10').validate(obj)
    return True


def get_tag_resource_types(api):
    endpoint_result = api.tag.get_tag_resource_types(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_resource_types(api):
    assert is_valid_get_tag_resource_types(
        get_tag_resource_types(api)
    )


def is_valid_get_tag_member_count(obj):
    json_schema_validate('jsd_2e9db85840fbb1cf_v1_2_10').validate(obj)
    return True


def get_tag_member_count(api):
    endpoint_result = api.tag.get_tag_member_count(
        id=get_tag(api).response[0].id,
        member_type=get_tag_resource_types(api).response[0],
        level='0',
        member_association_type=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_member_count(api):
    assert is_valid_get_tag_member_count(
        get_tag_member_count(api)
    )


def is_valid_updates_tag_membership(obj):
    json_schema_validate('jsd_45bc7a8344a8bc1e_v1_2_10').validate(obj)
    return True


def updates_tag_membership(api):
    tag = get_tag_created(api).response[0].id
    key = api.devices.get_device_list().response[0].id
    endpoint_result = api.tag.updates_tag_membership(
        memberToTags={key: [tag]},
        memberType='networkdevice',
        payload=None,
        active_validation=False
    )
    return endpoint_result


@pytest.mark.tag
def test_updates_tag_membership(api):
    assert is_valid_updates_tag_membership(
        updates_tag_membership(api)
    )


def is_valid_add_members_to_the_tag(obj):
    json_schema_validate('jsd_00a2fa6146089317_v1_2_10').validate(obj)
    return True


def add_members_to_the_tag(api):
    endpoint_result = api.tag.add_members_to_the_tag(
        id=get_tag_created(api).response[0].id,
        payload={
            "networkdevice": [api.devices.get_device_list().response[0].id]
        },
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_add_members_to_the_tag(api):
    assert is_valid_add_members_to_the_tag(
        add_members_to_the_tag(api)
    )


def is_valid_get_tag_members_by_id(obj):
    json_schema_validate('jsd_eab7abe048fb99ad_v1_2_10').validate(obj)
    return True


def get_tag_members_by_id(api):
    endpoint_result = api.tag.get_tag_members_by_id(
        id=get_tag_created(api).response[0].id,
        member_type=get_tag_resource_types(api).response[0],
        level='0',
        limit=None,
        member_association_type=None,
        offset=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_get_tag_members_by_id(api):
    assert is_valid_get_tag_members_by_id(
        get_tag_members_by_id(api)
    )


def is_valid_update_tag(obj):
    json_schema_validate('jsd_4d86a993469a9da9_v1_2_10').validate(obj)
    return True


def update_tag(api):
    tag = get_tag_created(api).response[0]
    endpoint_result = api.tag.update_tag(
        description=None,
        dynamicRules=None,
        id=tag.id,
        instanceTenantId=None,
        name='{} Updated'.format(tag.name),
        systemTag=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_update_tag(api):
    assert is_valid_update_tag(
        update_tag(api)
    )


def is_valid_remove_tag_member(obj):
    json_schema_validate('jsd_caa3ea704d78b37e_v1_2_10').validate(obj)
    return True


def remove_tag_member(api):
    tag = api.tag.get_tag(name='InterestingTool01 Updated').response[0]
    device = api.devices.get_device_list().response[0]
    endpoint_result = api.tag.remove_tag_member(
        id=tag.id,
        member_id=device.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_remove_tag_member(api):
    assert is_valid_remove_tag_member(
        remove_tag_member(api)
    )


def is_valid_delete_tag(obj):
    json_schema_validate('jsd_429c28154bdaa13d_v1_2_10').validate(obj)
    return True


def delete_tag(api):
    tag = api.tag.get_tag(name='InterestingTool01 Updated').response[0]
    endpoint_result = api.tag.delete_tag(
        id=tag.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.tag
def test_delete_tag(api):
    assert is_valid_delete_tag(
        delete_tag(api)
    )



# -*- coding: utf-8 -*-
"""DNACenterAPI pnp API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate
from tests.config import (NEW_VIRTUAL_ACCOUNT_PAYLOAD)


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_get_smart_account_list(obj):
    json_schema_validate('jsd_3cb24acb486b89d2_v1_2_10').validate(obj)
    return True


def get_smart_account_list(api):
    endpoint_result = api.pnp.get_smart_account_list(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_smart_account_list(api):
    assert is_valid_get_smart_account_list(
        get_smart_account_list(api)
    )


def is_valid_get_virtual_account_list(obj):
    json_schema_validate('jsd_70a479a6462a9496_v1_2_10').validate(obj)
    return True


def get_virtual_account_list(api):
    endpoint_result = api.pnp.get_virtual_account_list(
        domain=get_smart_account_list(api)[0],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_virtual_account_list(api):
    assert is_valid_get_virtual_account_list(
        get_virtual_account_list(api)
    )


def is_valid_add_virtual_account(obj):
    json_schema_validate('jsd_1e962af345b8b59f_v1_2_10').validate(obj)
    return True


def add_virtual_account(api):
    endpoint_result = api.pnp.add_virtual_account(
        autoSyncPeriod=None,
        ccoUser=None,
        expiry=None,
        lastSync=None,
        profile=None,
        smartAccountId=None,
        syncResult=None,
        syncResultStr=None,
        syncStartTime=None,
        syncStatus=None,
        tenantId=None,
        token=None,
        virtualAccountId=None,
        payload=NEW_VIRTUAL_ACCOUNT_PAYLOAD,
        active_validation=False
    )
    return endpoint_result


@pytest.mark.skipif(not all([NEW_VIRTUAL_ACCOUNT_PAYLOAD]) is True,
                    reason="tests.config values required not present")
@pytest.mark.pnp
def test_add_virtual_account(api):
    assert is_valid_add_virtual_account(
        add_virtual_account(api)
    )


def is_valid_get_sync_result_for_virtual_account(obj):
    json_schema_validate('jsd_0a9c988445cb91c8_v1_2_10').validate(obj)
    return True


def get_sync_result_for_virtual_account(api):
    endpoint_result = api.pnp.get_sync_result_for_virtual_account(
        domain=get_smart_account_list(api)[0],
        name=get_virtual_account_list(api)[0],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_sync_result_for_virtual_account(api):
    assert is_valid_get_sync_result_for_virtual_account(
        get_sync_result_for_virtual_account(api)
    )


def is_valid_update_pnp_server_profile(obj):
    json_schema_validate('jsd_6f9819e84178870c_v1_2_10').validate(obj)
    return True


def update_pnp_server_profile(api):
    endpoint_result = api.pnp.update_pnp_server_profile(
        autoSyncPeriod=None,
        ccoUser=None,
        expiry=None,
        lastSync=None,
        profile=None,
        smartAccountId=None,
        syncResult=None,
        syncResultStr=None,
        syncStartTime=None,
        syncStatus=None,
        tenantId=None,
        token=None,
        virtualAccountId=None,
        payload=get_sync_result_for_virtual_account(api),
        active_validation=False
    )
    return endpoint_result


@pytest.mark.pnp
def test_update_pnp_server_profile(api):
    assert is_valid_update_pnp_server_profile(
        update_pnp_server_profile(api)
    )


def is_valid_sync_virtual_account_devices(obj):
    json_schema_validate('jsd_a4b6c87a4ffb9efa_v1_2_10').validate(obj)
    return True


def sync_virtual_account_devices(api):
    endpoint_result = api.pnp.sync_virtual_account_devices(
        autoSyncPeriod=None,
        ccoUser=None,
        expiry=None,
        lastSync=None,
        profile=None,
        smartAccountId=None,
        syncResult=None,
        syncResultStr=None,
        syncStartTime=None,
        syncStatus=None,
        tenantId=None,
        token=None,
        virtualAccountId=None,
        payload=get_sync_result_for_virtual_account(api),
        active_validation=False
    )
    return endpoint_result


@pytest.mark.pnp
def test_sync_virtual_account_devices(api):
    assert is_valid_sync_virtual_account_devices(
        sync_virtual_account_devices(api)
    )


def is_valid_deregister_virtual_account(obj):
    json_schema_validate('jsd_2499e9ad42e8ae5b_v1_2_10').validate(obj)
    return True


def deregister_virtual_account(api):
    time.sleep(10)
    endpoint_result = api.pnp.deregister_virtual_account(
        domain=get_smart_account_list(api)[0],
        name=get_virtual_account_list(api)[0],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_deregister_virtual_account(api):
    assert is_valid_deregister_virtual_account(
        deregister_virtual_account(api)
    )


def is_valid_get_workflow_count(obj):
    json_schema_validate('jsd_7989f86846faaf99_v1_2_10').validate(obj)
    return True


def get_workflow_count(api):
    endpoint_result = api.pnp.get_workflow_count(
        name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_workflow_count(api):
    assert is_valid_get_workflow_count(
        get_workflow_count(api)
    )


def is_valid_get_workflows(obj):
    json_schema_validate('jsd_aeb4dad04a99bbe3_v1_2_10').validate(obj)
    return True


def get_workflows(api):
    endpoint_result = api.pnp.get_workflows(
        limit=None,
        name=None,
        offset=None,
        sort='addedOn',
        sort_order='des',
        type=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_workflows(api):
    assert is_valid_get_workflows(
        get_workflows(api)
    )


def is_valid_get_pnp_global_settings(obj):
    json_schema_validate('jsd_7e92f9eb46db8320_v1_2_10').validate(obj)
    return True


def get_pnp_global_settings(api):
    endpoint_result = api.pnp.get_pnp_global_settings(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_pnp_global_settings(api):
    assert is_valid_get_pnp_global_settings(
        get_pnp_global_settings(api)
    )


def is_valid_update_pnp_global_settings(obj):
    json_schema_validate('jsd_8da0391947088a5a_v1_2_10').validate(obj)
    return True


def update_pnp_global_settings(api):
    endpoint_result = api.pnp.update_pnp_global_settings(
        _id=None,
        aaaCredentials=None,
        acceptEula=None,
        defaultProfile=None,
        savaMappingList=None,
        taskTimeOuts=None,
        tenantId=None,
        version=None,
        payload=get_pnp_global_settings(api),
        active_validation=False
    )
    return endpoint_result


@pytest.mark.pnp
def test_update_pnp_global_settings(api):
    assert is_valid_update_pnp_global_settings(
        update_pnp_global_settings(api)
    )


def is_valid_get_device_count(obj):
    json_schema_validate('jsd_d9a1fa9c4068b23c_v1_2_10').validate(obj)
    return True


def get_device_count(api):
    endpoint_result = api.pnp.get_device_count(
        cm_state=None,
        last_contact=None,
        name=None,
        onb_state=None,
        pid=None,
        project_id=None,
        project_name=None,
        serial_number=None,
        smart_account_id=None,
        source=None,
        state=None,
        virtual_account_id=None,
        workflow_id=None,
        workflow_name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_device_count(api):
    assert is_valid_get_device_count(
        get_device_count(api)
    )


def is_valid_import_devices_in_bulk(obj):
    json_schema_validate('jsd_21a6db2540298f55_v1_2_10').validate(obj)
    return True


def import_devices_in_bulk(api):
    endpoint_result = api.pnp.import_devices_in_bulk(
        payload=[{
            "deviceInfo": {
                "serialNumber": "c3160d2650",
                "name": "Test device c3160d2650"
            }
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_import_devices_in_bulk(api):
    assert is_valid_import_devices_in_bulk(
        import_devices_in_bulk(api)
    )


def is_valid_get_device_list(obj):
    json_schema_validate('jsd_e6b3db8046c99654_v1_2_10').validate(obj)
    return True


def get_device_list(api):
    endpoint_result = api.pnp.get_device_list(
        cm_state=None,
        last_contact=None,
        limit=1,
        name=None,
        offset=None,
        onb_state=None,
        pid=None,
        project_id=None,
        project_name=None,
        serial_number=None,
        smart_account_id=None,
        sort=None,
        sort_order=None,
        source=None,
        state=None,
        virtual_account_id=None,
        workflow_id=None,
        workflow_name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_device_list(api):
    assert is_valid_get_device_list(
        get_device_list(api)
    )


def is_valid_get_device_by_id(obj):
    json_schema_validate('jsd_bab6c9e5440885cc_v1_2_10').validate(obj)
    return True


def get_device_by_id(api):
    endpoint_result = api.pnp.get_device_by_id(
        id=get_device_list(api)[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_device_by_id(api):
    assert is_valid_get_device_by_id(
        get_device_by_id(api)
    )


def is_valid_get_device_history(obj):
    json_schema_validate('jsd_f09319674049a7d4_v1_2_10').validate(obj)
    return True


def get_device_history(api):
    endpoint_result = api.pnp.get_device_history(
        serial_number=get_device_list(api)[0].deviceInfo.serialNumber,
        sort=None,
        sort_order=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_device_history(api):
    assert is_valid_get_device_history(
        get_device_history(api)
    )


def is_valid_add_device_2(obj):
    json_schema_validate('jsd_f3b26b5544cabab9_v1_2_10').validate(obj)
    return True


def add_device_2(api):
    endpoint_result = api.pnp.add_device(
        _id=None,
        deviceInfo={'serialNumber': 'FJC2048D0HX', 'name': 'catalyst_ap_test', 'pid': 'ISR4431-SEC/K9'},
        runSummaryList=None,
        systemResetWorkflow=None,
        systemWorkflow=None,
        tenantId=None,
        version=None,
        workflow=None,
        workflowParameters=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_add_device_2(api):
    assert is_valid_add_device_2(
        add_device_2(api)
    )


def is_valid_preview_config(obj):
    json_schema_validate('jsd_cf9418234d9ab37e_v1_2_10').validate(obj)
    return True


def preview_config(api):
    sites = api.sites.get_site_health(timestamp=None).response
    sites = list(filter(lambda x: x.siteType == 'building', sites))
    siteId = sites[-1].siteId if len(sites) > 0 else "1"
    endpoint_result = api.pnp.preview_config(
        deviceId=api.pnp.get_device_list(name='catalyst_ap_test')[0].id,
        siteId=siteId,
        type='Default',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_preview_config(api):
    assert is_valid_preview_config(
        preview_config(api)
    )


def is_valid_claim_a_device_to_a_site(obj):
    json_schema_validate('jsd_5889fb844939a13b_v1_2_10').validate(obj)
    return True


def claim_a_device_to_a_site(api):
    sites = api.sites.get_site_health(timestamp=None).response
    sites = list(filter(lambda x: x.siteType == 'building', sites))
    siteId = sites[-1].siteId if len(sites) > 0 else "1"
    deviceId = api.pnp.get_device_list(name='catalyst_ap_test')[0].id
    endpoint_result = api.pnp.claim_a_device_to_a_site(
        deviceId=None,
        siteId=None,
        type=None,
        payload={
            "siteId": siteId,
            "deviceId": deviceId,
            "type": "Default",
            "imageInfo": {"imageId": "", "skip": False},
            "configInfo": {"configId": "", "configParameters": []}
        },
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_claim_a_device_to_a_site(api):
    assert is_valid_claim_a_device_to_a_site(
        claim_a_device_to_a_site(api)
    )


def is_valid_delete_device_by_id_from_pnp_imported(obj):
    json_schema_validate('jsd_cdab9b474899ae06_v1_2_10').validate(obj)
    return True


def delete_device_by_id_from_pnp_imported(api):
    endpoint_result = api.pnp.delete_device_by_id_from_pnp(
        id=api.pnp.get_device_list(name='Test device c3160d2650')[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_delete_device_by_id_from_pnp_imported(api):
    assert is_valid_delete_device_by_id_from_pnp_imported(
        delete_device_by_id_from_pnp_imported(api)
    )


def is_valid_add_device(obj):
    json_schema_validate('jsd_f3b26b5544cabab9_v1_2_10').validate(obj)
    return True


def add_device(api):
    endpoint_result = api.pnp.add_device(
        _id=None,
        deviceInfo={'serialNumber': 'd2650c3160', 'name': 'Test device d2650c3160'},
        runSummaryList=None,
        systemResetWorkflow=None,
        systemWorkflow=None,
        tenantId=None,
        version=None,
        workflow=None,
        workflowParameters=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_add_device(api):
    assert is_valid_add_device(
        add_device(api)
    )


def is_valid_update_device(obj):
    json_schema_validate('jsd_09b0f9ce4239ae10_v1_2_10').validate(obj)
    return True


def update_device(api):
    endpoint_result = api.pnp.update_device(
        id=api.pnp.get_device_list(name='Test device d2650c3160')[0].id,
        _id=None,
        deviceInfo={'name': 'Test device d2650c3160-1'},
        runSummaryList=None,
        systemResetWorkflow=None,
        systemWorkflow=None,
        tenantId=None,
        version=None,
        workflow=None,
        workflowParameters=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_update_device(api):
    assert is_valid_update_device(
        update_device(api)
    )


def is_valid_add_a_workflow(obj):
    json_schema_validate('jsd_848b5a7b4f9b8c12_v1_2_10').validate(obj)
    return True


def add_a_workflow(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'Onboarding Configuration', projects))
    endpoint_result = api.template_programmer.create_template(
        project_id=filtered_project[0].id,
        composite=False,
        containingTemplates=[],
        deviceTypes=[
            {
                "productFamily": "Switches and Hubs",
            }
        ],
        name='test_template_05',
        rollbackTemplateContent='',
        rollbackTemplateParams=[],
        softwareType='IOS-XE',
        softwareVariant='XE',
        templateContent='show version\n',
        templateParams=[],
        active_validation=True
    )
    time.sleep(10)
    task = api.task.get_task_by_id(endpoint_result.response.taskId).response
    template = api.template_programmer.get_template_details(template_id=task.data)
    api.template_programmer.version_template(templateId=template.id)
    api.template_programmer.preview_template(templateId=template.id)
    endpoint_result = api.pnp.add_a_workflow(
        _id=None,
        addToInventory=None,
        addedOn=None,
        configId=None,
        currTaskIdx=None,
        description='test_devnet_1',
        endTime=None,
        execTime=None,
        imageId=None,
        instanceType=None,
        lastupdateOn=None,
        name='test_devnet_1',
        startTime=None,
        state=None,
        tasks=[{
            'taskSeqNo': 0,
            'name': 'Config Download',
            'type': 'Config',
            'startTime': 0,
            'endTime': 0,
            'timeTaken': 0,
            'currWorkItemIdx': 0,
            'configInfo': {
                'configId': template.id,
                'configFileUrl': None,
                'fileServiceId': None,
                'saveToStartUp': True,
                'connLossRollBack': True,
                'configParameters': None
            }
        }],
        tenantId=None,
        type='Standard',
        useState='Available',
        version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_add_a_workflow(api):
    assert is_valid_add_a_workflow(
        add_a_workflow(api)
    )


def is_valid_get_workflow_by_id(obj):
    json_schema_validate('jsd_80acb88e4ac9ac6d_v1_2_10').validate(obj)
    return True


def get_workflow_by_id(api):
    endpoint_result = api.pnp.get_workflow_by_id(
        id=get_workflows(api)[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_get_workflow_by_id(api):
    assert is_valid_get_workflow_by_id(
        get_workflow_by_id(api)
    )


def is_valid_update_workflow(obj):
    json_schema_validate('jsd_3086c9624f498b85_v1_2_10').validate(obj)
    return True


def update_workflow(api):
    workflow = api.pnp.get_workflows(name='test_devnet_1')[0]
    endpoint_result = api.pnp.update_workflow(
        id=workflow.id,
        _id=None,
        addToInventory=None,
        addedOn=None,
        configId=None,
        currTaskIdx=None,
        description=workflow.description,
        endTime=None,
        execTime=None,
        imageId=None,
        instanceType=None,
        lastupdateOn=None,
        name=workflow.name,
        startTime=None,
        state=None,
        tasks=workflow.tasks,
        tenantId=None,
        type=workflow.type,
        useState=workflow.useState,
        version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_update_workflow(api):
    assert is_valid_update_workflow(
        update_workflow(api)
    )


def is_valid_claim_device(obj):
    json_schema_validate('jsd_d8a619974a8a8c48_v1_2_10').validate(obj)
    return True


def claim_device(api):
    workflow = api.pnp.get_workflows(name='test_devnet_1')[0]
    configList = [workflow.tasks[0].configInfo]
    deviceId = api.pnp.get_device_list(name='catalyst_ap_test')[0].id
    endpoint_result = api.pnp.claim_device(
        configFileUrl=None,
        configId=None,
        deviceClaimList=[
            {
                "configList": configList,
                "deviceId": deviceId
            }
        ],
        fileServiceId=None,
        imageId=None,
        imageUrl=None,
        populateInventory=None,
        projectId=None,
        workflowId=workflow.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_claim_device(api):
    assert is_valid_claim_device(
        claim_device(api)
    )


def is_valid_un_claim_device(obj):
    json_schema_validate('jsd_0b836b7b4b6a9fd5_v1_2_10').validate(obj)
    return True


def un_claim_device(api):
    deviceId = api.pnp.get_device_list(name='catalyst_ap_test')[0].id
    endpoint_result = api.pnp.un_claim_device(
        deviceIdList=[deviceId],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_un_claim_device(api):
    assert is_valid_un_claim_device(
        un_claim_device(api)
    )


def is_valid_reset_device(obj):
    json_schema_validate('jsd_9e857b5a4a0bbcdb_v1_2_10').validate(obj)
    return True


def reset_device(api):
    workflow = api.pnp.get_workflows(name='test_devnet_1')[0]
    deviceId = api.pnp.get_device_list(name='catalyst_ap_test')[0].id
    try:
        endpoint_result = api.pnp.reset_device(
            deviceResetList=[{"deviceId": deviceId, "configList": [workflow.tasks[0].configInfo]}],
            projectId=None,
            workflowId=workflow.id,
            payload=None,
            active_validation=True
        )

    except Exception as e:
        assert(e.status_code == 400)
        assert('Invalid request for resetting' in e.message)
        endpoint_result = api.pnp._object_factory('', json_data={'message': e.message})
    return endpoint_result


@pytest.mark.pnp
def test_reset_device(api):
    assert is_valid_reset_device(
        reset_device(api)
    )


def is_valid_delete_workflow_by_id(obj):
    json_schema_validate('jsd_af8d7b0e470b8ae2_v1_2_10').validate(obj)
    return True


def delete_workflow_by_id(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'Onboarding Configuration', projects))
    templates = api.template_programmer.gets_the_templates_available(project_id=filtered_project[0].id)
    for template in templates:
        if 'test_template' in template.name:
            api.template_programmer.delete_template(template_id=template.templateId)
    endpoint_result = api.pnp.delete_workflow_by_id(
        id=api.pnp.get_workflows(name='test_devnet_1')[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_delete_workflow_by_id(api):
    assert is_valid_delete_workflow_by_id(
        delete_workflow_by_id(api)
    )


def is_valid_delete_device_by_id_from_pnp_added(obj):
    json_schema_validate('jsd_cdab9b474899ae06_v1_2_10').validate(obj)
    return True


def delete_device_by_id_from_pnp_added(api):
    endpoint_result = api.pnp.delete_device_by_id_from_pnp(
        id=api.pnp.get_device_list(name='Test device d2650c3160-1')[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_delete_device_by_id_from_pnp_added(api):
    assert is_valid_delete_device_by_id_from_pnp_added(
        delete_device_by_id_from_pnp_added(api)
    )


def is_valid_delete_device_by_id_from_pnp_added_2(obj):
    json_schema_validate('jsd_cdab9b474899ae06_v1_2_10').validate(obj)
    return True


def delete_device_by_id_from_pnp_added_2(api):
    endpoint_result = api.pnp.delete_device_by_id_from_pnp(
        id=api.pnp.get_device_list(name='catalyst_ap_test')[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.pnp
def test_delete_device_by_id_from_pnp_added_2(api):
    assert is_valid_delete_device_by_id_from_pnp_added_2(
        delete_device_by_id_from_pnp_added_2(api)
    )



# -*- coding: utf-8 -*-
"""DNACenterAPI devices API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate
from tests.config import MERAKI_ORG_ID


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_get_device_interface_count(obj):
    json_schema_validate('jsd_3d923b184dc9a4ca_v1_2_10').validate(obj)
    return True


def get_device_interface_count(api):
    endpoint_result = api.devices.get_device_interface_count(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_interface_count(api):
    assert is_valid_get_device_interface_count(
        get_device_interface_count(api)
    )


def is_valid_get_device_list(obj):
    json_schema_validate('jsd_20b19b52464b8972_v1_2_10').validate(obj)
    return True


def get_device_list(api):
    endpoint_result = api.devices.get_device_list(
        associated_wlc_ip=None,
        collection_interval=None,
        collection_status=None,
        error_code=None,
        error_description=None,
        family=None,
        hostname=None,
        id=None,
        license_name=None,
        license_status=None,
        license_type=None,
        location=None,
        location_name=None,
        mac_address=None,
        management_ip_address=None,
        module_equpimenttype=None,
        module_name=None,
        module_operationstatecode=None,
        module_partnumber=None,
        module_servicestate=None,
        module_vendorequipmenttype=None,
        not_synced_for_minutes=None,
        platform_id=None,
        reachability_status=None,
        role=None,
        serial_number=None,
        series=None,
        software_type=None,
        software_version=None,
        type=None,
        up_time=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_list(api):
    assert is_valid_get_device_list(
        get_device_list(api)
    )


def is_valid_sync_devices_using_forcesync(obj):
    json_schema_validate('jsd_3b9ef9674429be4c_v1_2_10').validate(obj)
    return True


def sync_devices_using_forcesync(api):
    endpoint_result = api.devices.sync_devices_using_forcesync(
        force_sync=None,
        payload=['get_device_list(api).response[0].id'],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_sync_devices_using_forcesync(api):
    assert is_valid_sync_devices_using_forcesync(
        sync_devices_using_forcesync(api)
    )


def is_valid_get_polling_interval_for_all_devices(obj):
    json_schema_validate('jsd_38bd0b884b89a785_v1_2_10').validate(obj)
    return True


def get_polling_interval_for_all_devices(api):
    endpoint_result = api.devices.get_polling_interval_for_all_devices(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_polling_interval_for_all_devices(api):
    assert is_valid_get_polling_interval_for_all_devices(
        get_polling_interval_for_all_devices(api)
    )


def is_valid_get_device_count(obj):
    json_schema_validate('jsd_5db21b8e43fab7d8_v1_2_10').validate(obj)
    return True


def get_device_count(api):
    endpoint_result = api.devices.get_device_count(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_count(api):
    assert is_valid_get_device_count(
        get_device_count(api)
    )


def is_valid_get_device_interface_vlans(obj):
    json_schema_validate('jsd_288df9494f2a9746_v1_2_10').validate(obj)
    return True


def get_device_interface_vlans(api):
    devices = api.devices.get_device_list().response
    devices_filtered = [device if 'Switches and Hubs' in device.family
                        and device.interfaceCount and device.interfaceCount > "1"
                        else None for device in devices]
    devices_filtered = list(filter(lambda x: x, devices_filtered))
    endpoint_result = api.devices.get_device_interface_vlans(
        id=devices_filtered[0].id,
        interface_type=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_interface_vlans(api):
    assert is_valid_get_device_interface_vlans(
        get_device_interface_vlans(api)
    )


def is_valid_get_device_interfaces_by_specified_range(obj):
    json_schema_validate('jsd_349c888443b89a58_v1_2_10').validate(obj)
    return True


def get_device_interfaces_by_specified_range(api):
    endpoint_result = api.devices.get_device_interfaces_by_specified_range(
        device_id=get_device_list(api).response[0].id,
        records_to_return=1,
        start_index=3,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_interfaces_by_specified_range(api):
    assert is_valid_get_device_interfaces_by_specified_range(
        get_device_interfaces_by_specified_range(api)
    )


def is_valid_get_device_config_by_id(obj):
    json_schema_validate('jsd_84b33a9e480abcaf_v1_2_10').validate(obj)
    return True


def get_device_config_by_id(api):
    endpoint_result = api.devices.get_device_config_by_id(
        network_device_id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_config_by_id(api):
    assert is_valid_get_device_config_by_id(
        get_device_config_by_id(api)
    )


def is_valid_get_device_config_count(obj):
    json_schema_validate('jsd_888f585c49b88441_v1_2_10').validate(obj)
    return True


def get_device_config_count(api):
    endpoint_result = api.devices.get_device_config_count(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_config_count(api):
    assert is_valid_get_device_config_count(
        get_device_config_count(api)
    )


def is_valid_get_all_interfaces(obj):
    json_schema_validate('jsd_f5947a4c439a8bf0_v1_2_10').validate(obj)
    return True


def get_all_interfaces(api):
    endpoint_result = api.devices.get_all_interfaces(
        limit=500,
        offset=1,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_all_interfaces(api):
    assert is_valid_get_all_interfaces(
        get_all_interfaces(api)
    )


def is_valid_get_interface_details(obj):
    json_schema_validate('jsd_4eb56a614cc9a2d2_v1_2_10').validate(obj)
    return True


def get_interface_details(api):
    endpoint_result = api.devices.get_interface_details(
        device_id=get_device_list(api).response[0].id,
        name=get_device_interface_vlans(api).response[0].interfaceName,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_interface_details(api):
    assert is_valid_get_interface_details(
        get_interface_details(api)
    )


def is_valid_get_polling_interval_by_id(obj):
    json_schema_validate('jsd_82918a1b4d289c5c_v1_2_10').validate(obj)
    return True


def get_polling_interval_by_id(api):
    endpoint_result = api.devices.get_polling_interval_by_id(
        id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_polling_interval_by_id(api):
    assert is_valid_get_polling_interval_by_id(
        get_polling_interval_by_id(api)
    )


def is_valid_get_module_count(obj):
    json_schema_validate('jsd_8db939744649a782_v1_2_10').validate(obj)
    return True


def get_module_count(api):
    endpoint_result = api.devices.get_module_count(
        device_id=get_device_list(api).response[0].id,
        name_list=None,
        operational_state_code_list=None,
        part_number_list=None,
        vendor_equipment_type_list=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_module_count(api):
    assert is_valid_get_module_count(
        get_module_count(api)
    )


def is_valid_get_interface_info_by_id(obj):
    json_schema_validate('jsd_ba9dc85b4b8a9a17_v1_2_10').validate(obj)
    return True


def get_interface_info_by_id(api):
    endpoint_result = api.devices.get_interface_info_by_id(
        device_id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_interface_info_by_id(api):
    assert is_valid_get_interface_info_by_id(
        get_interface_info_by_id(api)
    )


def is_valid_get_device_summary(obj):
    json_schema_validate('jsd_819f9aa54feab7bf_v1_2_10').validate(obj)
    return True


def get_device_summary(api):
    endpoint_result = api.devices.get_device_summary(
        id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_summary(api):
    assert is_valid_get_device_summary(
        get_device_summary(api)
    )


def is_valid_get_functional_capability_for_devices(obj):
    json_schema_validate('jsd_c3b3c9ef4e6b8a09_v1_2_10').validate(obj)
    return True


def get_functional_capability_for_devices(api):
    endpoint_result = api.devices.get_functional_capability_for_devices(
        device_id=get_device_list(api).response[0].id,
        function_name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_functional_capability_for_devices(api):
    assert is_valid_get_functional_capability_for_devices(
        get_functional_capability_for_devices(api)
    )


def is_valid_get_device_interface_count_by_id(obj):
    json_schema_validate('jsd_5b8639224cd88ea7_v1_2_10').validate(obj)
    return True


def get_device_interface_count_by_id(api):
    endpoint_result = api.devices.get_device_interface_count_by_id(
        device_id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_interface_count_by_id(api):
    assert is_valid_get_device_interface_count_by_id(
        get_device_interface_count_by_id(api)
    )


def is_valid_get_modules(obj):
    json_schema_validate('jsd_eb8249e34f69b0f1_v1_2_10').validate(obj)
    return True


def get_modules(api):
    endpoint_result = api.devices.get_modules(
        device_id=get_device_list(api).response[0].id,
        limit=None,
        name_list=None,
        offset=None,
        operational_state_code_list=None,
        part_number_list=None,
        vendor_equipment_type_list=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_modules(api):
    assert is_valid_get_modules(
        get_modules(api)
    )


def is_valid_get_wireless_lan_controller_details_by_id(obj):
    json_schema_validate('jsd_f6826a8e41bba242_v1_2_10').validate(obj)
    return True


def get_wireless_lan_controller_details_by_id(api):
    endpoint_result = api.devices.get_wireless_lan_controller_details_by_id(
        id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_wireless_lan_controller_details_by_id(api):
    assert is_valid_get_wireless_lan_controller_details_by_id(
        get_wireless_lan_controller_details_by_id(api)
    )


def is_valid_get_organization_list_for_meraki(obj):
    json_schema_validate('jsd_84b37ae54c59ab28_v1_2_10').validate(obj)
    return True


def get_organization_list_for_meraki(api):
    sub_type = "SNMPV2_READ_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    device_list_ips = [d.managementIpAddress for d in api.devices.get_device_list().response]
    device_ip = device_list_ips[0]
    a = list(filter(lambda x: x.rsplit('.', 1)[0] == device_ip.rsplit('.', 1)[0], device_list_ips))
    b = set(range(2, 103)) - set([int(x.rsplit('.', 1)[-1]) for x in a])
    new_ipAddress = device_ip.rsplit('.', 1)[0] + '.' + str(list(b)[0])
    api.devices.add_device(
        cliTransport="ssh",
        enablePassword="false",
        ipAddress=[new_ipAddress],
        merakiOrgId=MERAKI_ORG_ID,
        password="W.~&KV9ha",
        snmpMode="NOAUTHNOPRIV",
        snmpROCommunity=credentials[0].id,
        snmpRWCommunity="",
        snmpRetry=0,
        snmpTimeout=0,
        snmpUserName="test_user_devnet",
        snmpVersion="v2",
        type="NETWORK_DEVICE",
        userName="test_user_devnet",
        active_validation=True
    )
    time.sleep(10)
    filtered_device_list = api.devices.get_device_list(
        family='null', hostname='null', software_type='null',
        management_ip_address=new_ipAddress
    ).response
    endpoint_result = api.devices.get_organization_list_for_meraki(
        id=filtered_device_list[0].id,
        payload=None,
        active_validation=True
    )
    time.sleep(10)
    api.devices.delete_device_by_id(
        id=filtered_device_list[0].id,
        is_force_delete=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.skipif(not all([MERAKI_ORG_ID]) is True,
                    reason="tests.config values required not present")
@pytest.mark.devices
def test_get_organization_list_for_meraki(api):
    assert is_valid_get_organization_list_for_meraki(
        get_organization_list_for_meraki(api)
    )


def is_valid_get_ospf_interfaces(obj):
    json_schema_validate('jsd_70ad397649e9b4d3_v1_2_10').validate(obj)
    return True


def get_ospf_interfaces(api):
    endpoint_result = api.devices.get_ospf_interfaces(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_ospf_interfaces(api):
    assert is_valid_get_ospf_interfaces(
        get_ospf_interfaces(api)
    )


def is_valid_get_functional_capability_by_id(obj):
    json_schema_validate('jsd_81bb4804405a8d2f_v1_2_10').validate(obj)
    return True


def get_functional_capability_by_id(api):
    functional_capability_devices = get_functional_capability_for_devices(api).response
    functional_capability = functional_capability_devices[0].functionalCapability[0]
    endpoint_result = api.devices.get_functional_capability_by_id(
        id=functional_capability.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_functional_capability_by_id(api):
    assert is_valid_get_functional_capability_by_id(
        get_functional_capability_by_id(api)
    )


def is_valid_get_isis_interfaces(obj):
    json_schema_validate('jsd_84ad8b0e42cab48a_v1_2_10').validate(obj)
    return True


def get_isis_interfaces(api):
    endpoint_result = api.devices.get_isis_interfaces(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_isis_interfaces(api):
    assert is_valid_get_isis_interfaces(
        get_isis_interfaces(api)
    )


def is_valid_get_device_config_for_all_devices(obj):
    json_schema_validate('jsd_b7bcaa084e2b90d0_v1_2_10').validate(obj)
    return True


def get_device_config_for_all_devices(api):
    endpoint_result = api.devices.get_device_config_for_all_devices(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_config_for_all_devices(api):
    assert is_valid_get_device_config_for_all_devices(
        get_device_config_for_all_devices(api)
    )


def is_valid_get_interface_by_ip(obj):
    json_schema_validate('jsd_cd8469e647caab0e_v1_2_10').validate(obj)
    return True


def get_interface_by_ip(api):
    interfaces = get_all_interfaces(api).response
    filtered_interfaces = list(filter(lambda x: x.ipv4Address is not None, interfaces))
    endpoint_result = api.devices.get_interface_by_ip(
        ip_address=filtered_interfaces[0].ipv4Address,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_interface_by_ip(api):
    assert is_valid_get_interface_by_ip(
        get_interface_by_ip(api)
    )


def is_valid_get_network_device_by_ip(obj):
    json_schema_validate('jsd_d0a4b88145aabb51_v1_2_10').validate(obj)
    return True


def get_network_device_by_ip(api):
    endpoint_result = api.devices.get_network_device_by_ip(
        ip_address=get_device_list(api).response[0].managementIpAddress,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_network_device_by_ip(api):
    assert is_valid_get_network_device_by_ip(
        get_network_device_by_ip(api)
    )


def is_valid_get_device_by_id(obj):
    json_schema_validate('jsd_8fa8eb404a4a8d96_v1_2_10').validate(obj)
    return True


def get_device_by_id(api):
    endpoint_result = api.devices.get_device_by_id(
        id=get_device_list(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_by_id(api):
    assert is_valid_get_device_by_id(
        get_device_by_id(api)
    )


def is_valid_sync_devices(obj):
    json_schema_validate('jsd_aeb9eb67460b92df_v1_2_10').validate(obj)
    return True


def sync_devices(api):
    endpoint_result = api.devices.sync_devices(
        cliTransport=None,
        computeDevice=None,
        enablePassword=None,
        extendedDiscoveryInfo=None,
        httpPassword=None,
        httpPort=None,
        httpSecure=None,
        httpUserName=None,
        ipAddress=None,
        merakiOrgId=None,
        netconfPort=None,
        password=None,
        serialNumber=None,
        snmpAuthPassphrase=None,
        snmpAuthProtocol=None,
        snmpMode=None,
        snmpPrivPassphrase=None,
        snmpPrivProtocol=None,
        snmpROCommunity=None,
        snmpRWCommunity=None,
        snmpRetry=None,
        snmpTimeout=None,
        snmpUserName=None,
        snmpVersion=None,
        type=None,
        updateMgmtIPaddressList=None,
        userName=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_sync_devices(api):
    assert is_valid_sync_devices(
        sync_devices(api)
    )


def is_valid_get_interface_by_id(obj):
    json_schema_validate('jsd_b888792d43baba46_v1_2_10').validate(obj)
    return True


def get_interface_by_id(api):
    endpoint_result = api.devices.get_interface_by_id(
        id=get_all_interfaces(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_interface_by_id(api):
    assert is_valid_get_interface_by_id(
        get_interface_by_id(api)
    )


def is_valid_get_device_by_serial_number(obj):
    json_schema_validate('jsd_d888ab6d4d59a8c1_v1_2_10').validate(obj)
    return True


def get_device_by_serial_number(api):
    device_list = get_device_list(api).response
    filtered_device_list = list(filter(lambda x: x.serialNumber, device_list))
    endpoint_result = api.devices.get_device_by_serial_number(
        serial_number=filtered_device_list[0].serialNumber,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_by_serial_number(api):
    assert is_valid_get_device_by_serial_number(
        get_device_by_serial_number(api)
    )


def is_valid_get_network_device_by_pagination_range(obj):
    json_schema_validate('jsd_f49548c54be8a3e2_v1_2_10').validate(obj)
    return True


def get_network_device_by_pagination_range(api):
    endpoint_result = api.devices.get_network_device_by_pagination_range(
        records_to_return=4,
        start_index=1,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_network_device_by_pagination_range(api):
    assert is_valid_get_network_device_by_pagination_range(
        get_network_device_by_pagination_range(api)
    )


def is_valid_retrieves_all_network_devices(obj):
    json_schema_validate('jsd_ffa748cc44e9a437_v1_2_10').validate(obj)
    return True


def retrieves_all_network_devices(api):
    endpoint_result = api.devices.retrieves_all_network_devices(
        associated_wlc_ip=None,
        collection_interval=None,
        collection_status=None,
        error_code=None,
        family=None,
        hostname=get_device_list(api).response[0].hostname,
        limit=None,
        mac_address=None,
        management_ip_address=None,
        offset=None,
        platform_id=None,
        reachability_failure_reason=None,
        reachability_status=None,
        role=None,
        role_source=None,
        serial_number=None,
        series=None,
        software_type=None,
        software_version=None,
        type=None,
        up_time=None,
        vrf_name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_retrieves_all_network_devices(api):
    assert is_valid_retrieves_all_network_devices(
        retrieves_all_network_devices(api)
    )


def is_valid_get_device_detail(obj):
    json_schema_validate('jsd_89b2fb144f5bb09b_v1_2_10').validate(obj)
    return True


def get_device_detail(api):
    endpoint_result = api.devices.get_device_detail(
        identifier='macAddress',
        search_by=get_device_list(api).response[0].macAddress,
        timestamp=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_detail(api):
    assert is_valid_get_device_detail(
        get_device_detail(api)
    )


def is_valid_get_module_info_by_id(obj):
    json_schema_validate('jsd_0db7da744c0b83d8_v1_2_10').validate(obj)
    return True


def get_module_info_by_id(api):
    endpoint_result = api.devices.get_module_info_by_id(
        id=get_device_list(api).response[0].lineCardId.split(',')[0],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_module_info_by_id(api):
    assert is_valid_get_module_info_by_id(
        get_module_info_by_id(api)
    )


def is_valid_add_device(obj):
    json_schema_validate('jsd_4bb22af046fa8f08_v1_2_10').validate(obj)
    return True


def add_device(api):
    sub_type = "SNMPV2_READ_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    device_list_ips = [d.managementIpAddress for d in api.devices.get_device_list().response]
    device_ip = device_list_ips[0]
    a = list(filter(lambda x: x.rsplit('.', 1)[0] == device_ip.rsplit('.', 1)[0], device_list_ips))
    b = set(range(2, 103)) - set([int(x.rsplit('.', 1)[-1]) for x in a])
    new_ipAddress = device_ip.rsplit('.', 1)[0] + '.' + str(list(b)[0])
    endpoint_result = api.devices.add_device(
        cliTransport=None,
        computeDevice=None,
        enablePassword=None,
        extendedDiscoveryInfo=None,
        httpPassword=None,
        httpPort=None,
        httpSecure=None,
        httpUserName=None,
        ipAddress=[new_ipAddress],
        merakiOrgId=None,
        netconfPort=None,
        password=None,
        serialNumber=None,
        snmpAuthPassphrase=None,
        snmpAuthProtocol=None,
        snmpMode=None,
        snmpPrivPassphrase=None,
        snmpPrivProtocol=None,
        snmpROCommunity=None,
        snmpRWCommunity=None,
        snmpRetry=None,
        snmpTimeout=None,
        snmpUserName=None,
        snmpVersion=None,
        type=None,
        updateMgmtIPaddressList=None,
        userName=None,
        payload={
            "cliTransport": "ssh",
            "enablePassword": "false",
            "snmpMode": "NOAUTHNOPRIV",
            "snmpROCommunity": credentials[0].id,
            "snmpRWCommunity": "",
            "snmpRetry": 0,
            "snmpTimeout": 0,
            "snmpUserName": "test_user_devnet",
            "snmpVersion": "v2",
            "type": "NETWORK_DEVICE",
            "userName": "test_user_devnet",
            "password": "W.~&KV9ha"
        },
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_add_device(api):
    assert is_valid_add_device(
        add_device(api)
    )


def is_valid_get_device_by_id_last(obj):
    json_schema_validate('jsd_8fa8eb404a4a8d96_v1_2_10').validate(obj)
    return True


def get_device_by_id_last(api):
    endpoint_result = api.devices.get_device_by_id(
        id=get_device_list(api).response[-1].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_get_device_by_id_last(api):
    assert is_valid_get_device_by_id_last(
        get_device_by_id_last(api)
    )


def is_valid_update_device_role(obj):
    json_schema_validate('jsd_b9855ad54ae98156_v1_2_10').validate(obj)
    return True


def update_device_role(api):
    endpoint_result = api.devices.update_device_role(
        id=get_device_list(api).response[0].id,
        role=get_device_list(api).response[0].role,
        roleSource=get_device_list(api).response[0].roleSource,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_update_device_role(api):
    assert is_valid_update_device_role(
        update_device_role(api)
    )


def is_valid_register_device_for_wsa(obj):
    json_schema_validate('jsd_c9809b6744f8a502_v1_2_10').validate(obj)
    return True


def register_device_for_wsa(api):
    endpoint_result = api.devices.register_device_for_wsa(
        macaddress=get_device_list(api).response[0].macAddress,
        serial_number=get_device_list(api).response[0].serialNumber,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_register_device_for_wsa(api):
    assert is_valid_register_device_for_wsa(
        register_device_for_wsa(api)
    )


def is_valid_export_device_list(obj):
    json_schema_validate('jsd_cd98780f4888a66d_v1_2_10').validate(obj)
    return True


def export_device_list(api):
    endpoint_result = api.devices.export_device_list(
        deviceUuids=[get_device_list(api).response[0].id],
        id=get_device_list(api).response[0].id,
        operationEnum='CREDENTIALDETAILS',
        parameters=None,
        password=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_export_device_list(api):
    assert is_valid_export_device_list(
        export_device_list(api)
    )


def is_valid_delete_device_by_id(obj):
    json_schema_validate('jsd_1c894b5848eab214_v1_2_10').validate(obj)
    return True


def delete_device_by_id(api):
    filtered_device_list = api.devices.get_device_list(
        family='null', hostname='null', software_type='null'
    ).response
    endpoint_result = api.devices.delete_device_by_id(
        id=filtered_device_list[0].id,
        is_force_delete=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.devices
def test_delete_device_by_id(api):
    assert is_valid_delete_device_by_id(
        delete_device_by_id(api)
    )



# -*- coding: utf-8 -*-
"""Test suite for the community-developed Python SDK for the DNA Center APIs.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import dnacentersdk
from tests.environment import (
    DNA_CENTER_USERNAME, DNA_CENTER_PASSWORD,
    DNA_CENTER_ENCODED_AUTH, DNA_CENTER_VERSION,
)

from dnacentersdk.api.authentication import Authentication
from dnacentersdk.api.v1_2_10.clients import \
    Clients as Clients_v1_2_10
from dnacentersdk.api.v1_2_10.command_runner import \
    CommandRunner as CommandRunner_v1_2_10
from dnacentersdk.api.v1_2_10.devices import \
    Devices as Devices_v1_2_10
from dnacentersdk.api.v1_2_10.fabric_wired import \
    FabricWired as FabricWired_v1_2_10
from dnacentersdk.api.v1_2_10.file import \
    File as File_v1_2_10
from dnacentersdk.api.v1_2_10.network_discovery import \
    NetworkDiscovery as NetworkDiscovery_v1_2_10
from dnacentersdk.api.v1_2_10.networks import \
    Networks as Networks_v1_2_10
from dnacentersdk.api.v1_2_10.non_fabric_wireless import \
    NonFabricWireless as NonFabricWireless_v1_2_10
from dnacentersdk.api.v1_2_10.path_trace import \
    PathTrace as PathTrace_v1_2_10
from dnacentersdk.api.v1_2_10.pnp import \
    Pnp as Pnp_v1_2_10
from dnacentersdk.api.v1_2_10.swim import \
    Swim as Swim_v1_2_10
from dnacentersdk.api.v1_2_10.site_profile import \
    SiteProfile as SiteProfile_v1_2_10
from dnacentersdk.api.v1_2_10.sites import \
    Sites as Sites_v1_2_10
from dnacentersdk.api.v1_2_10.tag import \
    Tag as Tag_v1_2_10
from dnacentersdk.api.v1_2_10.task import \
    Task as Task_v1_2_10
from dnacentersdk.api.v1_2_10.template_programmer import \
    TemplateProgrammer as TemplateProgrammer_v1_2_10
from dnacentersdk.api.v1_3_0.clients import \
    Clients as Clients_v1_3_0
from dnacentersdk.api.v1_3_0.command_runner import \
    CommandRunner as CommandRunner_v1_3_0
from dnacentersdk.api.v1_3_0.devices import \
    Devices as Devices_v1_3_0
from dnacentersdk.api.v1_3_0.fabric_wired import \
    FabricWired as FabricWired_v1_3_0
from dnacentersdk.api.v1_3_0.file import \
    File as File_v1_3_0
from dnacentersdk.api.v1_3_0.network_discovery import \
    NetworkDiscovery as NetworkDiscovery_v1_3_0
from dnacentersdk.api.v1_3_0.networks import \
    Networks as Networks_v1_3_0
from dnacentersdk.api.v1_3_0.non_fabric_wireless import \
    NonFabricWireless as NonFabricWireless_v1_3_0
from dnacentersdk.api.v1_3_0.path_trace import \
    PathTrace as PathTrace_v1_3_0
from dnacentersdk.api.v1_3_0.pnp import \
    Pnp as Pnp_v1_3_0
from dnacentersdk.api.v1_3_0.swim import \
    Swim as Swim_v1_3_0
from dnacentersdk.api.v1_3_0.site_profile import \
    SiteProfile as SiteProfile_v1_3_0
from dnacentersdk.api.v1_3_0.sites import \
    Sites as Sites_v1_3_0
from dnacentersdk.api.v1_3_0.tag import \
    Tag as Tag_v1_3_0
from dnacentersdk.api.v1_3_0.task import \
    Task as Task_v1_3_0
from dnacentersdk.api.v1_3_0.template_programmer import \
    TemplateProgrammer as TemplateProgrammer_v1_3_0
from dnacentersdk.api.custom_caller import CustomCaller

from tests.config import (
    DEFAULT_BASE_URL, DEFAULT_VERIFY,
    DEFAULT_SINGLE_REQUEST_TIMEOUT, DEFAULT_WAIT_ON_RATE_LIMIT
)
import pytest


@pytest.mark.dnacentersdk
class TestDNACenterSDK:
    """Test the package-level code."""

    def test_package_contents(self):
        """Ensure the package contains the correct top-level objects."""
        # DNA Center API Wrapper
        assert hasattr(dnacentersdk, "DNACenterAPI")

        # Exceptions
        assert hasattr(dnacentersdk, "AccessTokenError")
        assert hasattr(dnacentersdk, "ApiError")
        assert hasattr(dnacentersdk, "dnacentersdkException")
        assert hasattr(dnacentersdk, "MalformedRequest")
        assert hasattr(dnacentersdk, "RateLimitError")
        assert hasattr(dnacentersdk, "RateLimitWarning")
        assert hasattr(dnacentersdk, "VersionError")

        # Data Models
        assert hasattr(dnacentersdk, "mydict_data_factory")
        assert hasattr(dnacentersdk, "json_schema_validate")

    @pytest.mark.dnacentersdk
    def test_default_base_url(self, api):
        assert api.base_url == DEFAULT_BASE_URL

    @pytest.mark.dnacentersdk
    def test_custom_base_url(self):
        custom_url = "https://custom.domain.com/v1/"
        with pytest.raises(dnacentersdk.exceptions.ApiError):
            dnacentersdk.DNACenterAPI(username=DNA_CENTER_USERNAME,
                                      password=DNA_CENTER_PASSWORD,
                                      encoded_auth=DNA_CENTER_ENCODED_AUTH,
                                      base_url=custom_url,
                                      verify=DEFAULT_VERIFY,
                                      version=DNA_CENTER_VERSION)

    @pytest.mark.dnacentersdk
    def test_default_single_request_timeout(self, api):
        assert api.single_request_timeout == \
            DEFAULT_SINGLE_REQUEST_TIMEOUT

    @pytest.mark.dnacentersdk
    def test_custom_single_request_timeout(self):
        custom_timeout = 10
        connection_object = dnacentersdk.DNACenterAPI(username=DNA_CENTER_USERNAME,
                                                      password=DNA_CENTER_PASSWORD,
                                                      encoded_auth=DNA_CENTER_ENCODED_AUTH,
                                                      base_url=DEFAULT_BASE_URL,
                                                      single_request_timeout=custom_timeout,
                                                      verify=DEFAULT_VERIFY,
                                                      version=DNA_CENTER_VERSION)
        assert connection_object.single_request_timeout == custom_timeout

    @pytest.mark.dnacentersdk
    def test_default_wait_on_rate_limit(self, api):
        assert api.wait_on_rate_limit == \
            DEFAULT_WAIT_ON_RATE_LIMIT

    @pytest.mark.dnacentersdk
    def test_non_default_wait_on_rate_limit(self):
        connection_object = dnacentersdk.DNACenterAPI(username=DNA_CENTER_USERNAME,
                                                      password=DNA_CENTER_PASSWORD,
                                                      encoded_auth=DNA_CENTER_ENCODED_AUTH,
                                                      base_url=DEFAULT_BASE_URL,
                                                      wait_on_rate_limit=not DEFAULT_WAIT_ON_RATE_LIMIT,
                                                      verify=DEFAULT_VERIFY,
                                                      version=DNA_CENTER_VERSION)
        assert connection_object.wait_on_rate_limit != \
            DEFAULT_WAIT_ON_RATE_LIMIT

    @pytest.mark.dnacentersdk
    def test_api_object_creation(self, api):
        assert isinstance(api.authentication, Authentication)
        assert isinstance(api.custom_caller, CustomCaller)
        if api.version == '1.2.10':
            assert isinstance(api.clients, Clients_v1_2_10)
            assert isinstance(api.command_runner, CommandRunner_v1_2_10)
            assert isinstance(api.devices, Devices_v1_2_10)
            assert isinstance(api.fabric_wired, FabricWired_v1_2_10)
            assert isinstance(api.file, File_v1_2_10)
            assert isinstance(api.network_discovery, NetworkDiscovery_v1_2_10)
            assert isinstance(api.networks, Networks_v1_2_10)
            assert isinstance(api.non_fabric_wireless, NonFabricWireless_v1_2_10)
            assert isinstance(api.path_trace, PathTrace_v1_2_10)
            assert isinstance(api.pnp, Pnp_v1_2_10)
            assert isinstance(api.swim, Swim_v1_2_10)
            assert isinstance(api.site_profile, SiteProfile_v1_2_10)
            assert isinstance(api.sites, Sites_v1_2_10)
            assert isinstance(api.tag, Tag_v1_2_10)
            assert isinstance(api.task, Task_v1_2_10)
            assert isinstance(api.template_programmer, TemplateProgrammer_v1_2_10)
        if api.version == '1.3.0':
            assert isinstance(api.clients, Clients_v1_3_0)
            assert isinstance(api.command_runner, CommandRunner_v1_3_0)
            assert isinstance(api.devices, Devices_v1_3_0)
            assert isinstance(api.fabric_wired, FabricWired_v1_3_0)
            assert isinstance(api.file, File_v1_3_0)
            assert isinstance(api.network_discovery, NetworkDiscovery_v1_3_0)
            assert isinstance(api.networks, Networks_v1_3_0)
            assert isinstance(api.non_fabric_wireless, NonFabricWireless_v1_3_0)
            assert isinstance(api.path_trace, PathTrace_v1_3_0)
            assert isinstance(api.pnp, Pnp_v1_3_0)
            assert isinstance(api.swim, Swim_v1_3_0)
            assert isinstance(api.site_profile, SiteProfile_v1_3_0)
            assert isinstance(api.sites, Sites_v1_3_0)
            assert isinstance(api.tag, Tag_v1_3_0)
            assert isinstance(api.task, Task_v1_3_0)
            assert isinstance(api.template_programmer, TemplateProgrammer_v1_3_0)



# -*- coding: utf-8 -*-
"""DNACenterAPI network_discovery API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_delete_all_discovery(obj):
    json_schema_validate('jsd_db8e09234a988bab_v1_2_10').validate(obj)
    return True


def delete_all_discovery(api):
    endpoint_result = api.network_discovery.delete_all_discovery(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_all_discovery(api):
    assert is_valid_delete_all_discovery(
        delete_all_discovery(api)
    )


def is_valid_create_cli_credentials(obj):
    json_schema_validate('jsd_948ea8194348bc0b_v1_2_10').validate(obj)
    return True


def create_cli_credentials(api):
    endpoint_result = api.network_discovery.create_cli_credentials(
        payload=[{
            "username": "test_user_devnet",
            "password": "NO!$DATA!$"
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_cli_credentials(api):
    assert is_valid_create_cli_credentials(
        create_cli_credentials(api)
    )


def is_valid_create_netconf_credentials(obj):
    json_schema_validate('jsd_17929bc7465bb564_v1_2_10').validate(obj)
    return True


def create_netconf_credentials(api):
    endpoint_result = api.network_discovery.create_netconf_credentials(
        payload=[{
            "netconfPort": '65533'  # range of 1 to 65535
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_netconf_credentials(api):
    assert is_valid_create_netconf_credentials(
        create_netconf_credentials(api)
    )


def is_valid_create_snmp_write_community(obj):
    json_schema_validate('jsd_6bacb8d14639bdc7_v1_2_10').validate(obj)
    return True


def create_snmp_write_community(api):
    endpoint_result = api.network_discovery.create_snmp_write_community(
        payload=[{
            "writeCommunity": "NO!$DATA!$",
            "description": "created snmpv2"
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_snmp_write_community(api):
    assert is_valid_create_snmp_write_community(
        create_snmp_write_community(api)
    )


def is_valid_create_snmp_read_community(obj):
    json_schema_validate('jsd_7aa3da9d4e098ef2_v1_2_10').validate(obj)
    return True


def create_snmp_read_community(api):
    endpoint_result = api.network_discovery.create_snmp_read_community(
        payload=[{
            "readCommunity": "NO!$DATA!$",
            "description": "created snmpv2"
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_snmp_read_community(api):
    assert is_valid_create_snmp_read_community(
        create_snmp_read_community(api)
    )


def is_valid_create_http_write_credentials(obj):
    json_schema_validate('jsd_4d9ca8e2431a8a24_v1_2_10').validate(obj)
    return True


def create_http_write_credentials(api):
    endpoint_result = api.network_discovery.create_http_write_credentials(
        payload=[{
            "username": "test_user_devnet",
            "password": "W.~&KV9ha",
            "port": 8080
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_http_write_credentials(api):
    assert is_valid_create_http_write_credentials(
        create_http_write_credentials(api)
    )


def is_valid_create_http_read_credentials(obj):
    json_schema_validate('jsd_bf859ac64a0ba19c_v1_2_10').validate(obj)
    return True


def create_http_read_credentials(api):
    endpoint_result = api.network_discovery.create_http_read_credentials(
        payload=[{
            "username": "test_user_devnet",
            "password": "W.~&KV9ha",
            "port": 8080
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_http_read_credentials(api):
    assert is_valid_create_http_read_credentials(
        create_http_read_credentials(api)
    )


def is_valid_create_update_snmp_properties(obj):
    json_schema_validate('jsd_a5ac99774c6bb541_v1_2_10').validate(obj)
    return True


def create_update_snmp_properties(api):
    endpoint_result = api.network_discovery.create_update_snmp_properties(
        payload=[{
            "intValue": 1,
            "systemPropertyName": "version"
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_update_snmp_properties(api):
    assert is_valid_create_update_snmp_properties(
        create_update_snmp_properties(api)
    )


def is_valid_create_snmpv3_credentials(obj):
    json_schema_validate('jsd_979688084b7ba60d_v1_2_10').validate(obj)
    return True


def create_snmpv3_credentials(api):
    endpoint_result = api.network_discovery.create_snmpv3_credentials(
        payload=[{
            "snmpMode": "NOAUTHNOPRIV",
            "username": "test_user_devnet"
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_create_snmpv3_credentials(api):
    assert is_valid_create_snmpv3_credentials(
        create_snmpv3_credentials(api)
    )


def is_valid_get_credential_sub_type_by_credential_id(obj):
    json_schema_validate('jsd_58a3699e489b9529_v1_2_10').validate(obj)
    return True


def get_credential_sub_type_by_credential_id(api):
    endpoint_result = api.network_discovery.get_credential_sub_type_by_credential_id(
        id=get_global_credentials(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_credential_sub_type_by_credential_id(api):
    assert is_valid_get_credential_sub_type_by_credential_id(
        get_credential_sub_type_by_credential_id(api)
    )


def is_valid_get_snmp_properties(obj):
    json_schema_validate('jsd_44974ba5435a801d_v1_2_10').validate(obj)
    return True


def get_snmp_properties(api):
    endpoint_result = api.network_discovery.get_snmp_properties(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_snmp_properties(api):
    assert is_valid_get_snmp_properties(
        get_snmp_properties(api)
    )


def is_valid_start_discovery(obj):
    json_schema_validate('jsd_55b439dc4239b140_v1_2_10').validate(obj)
    return True


def start_discovery(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type='CLI').response
    credentialIdList = [x.id for x in credentials]
    endpoint_result = api.network_discovery.start_discovery(
        cdpLevel=16,
        discoveryType='CDP',
        enablePasswordList=None,
        globalCredentialIdList=credentialIdList[0:5],
        httpReadCredential=None,
        httpWriteCredential=None,
        ipAddressList='10.10.22.22',
        ipFilterList=None,
        lldpLevel=None,
        name='start_discovery_test',
        netconfPort='65535',
        noAddNewDevice=None,
        parentDiscoveryId=None,
        passwordList=None,
        preferredMgmtIPMethod=None,
        protocolOrder='ssh',
        reDiscovery=None,
        retry=None,
        snmpAuthPassphrase=None,
        snmpAuthProtocol=None,
        snmpMode=None,
        snmpPrivPassphrase=None,
        snmpPrivProtocol=None,
        snmpROCommunity=None,
        snmpROCommunityDesc=None,
        snmpRWCommunity=None,
        snmpRWCommunityDesc=None,
        snmpUserName=None,
        snmpVersion=None,
        timeout=None,
        updateMgmtIp=None,
        userNameList=None,
        payload=None,
        active_validation=True
    )
    time.sleep(10)
    return endpoint_result


@pytest.mark.network_discovery
def test_start_discovery(api):
    assert is_valid_start_discovery(
        start_discovery(api)
    )


def is_valid_get_count_of_all_discovery_jobs(obj):
    json_schema_validate('jsd_069d9823451b892d_v1_2_10').validate(obj)
    return True


def get_count_of_all_discovery_jobs(api):
    endpoint_result = api.network_discovery.get_count_of_all_discovery_jobs(
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_count_of_all_discovery_jobs(api):
    assert is_valid_get_count_of_all_discovery_jobs(
        get_count_of_all_discovery_jobs(api)
    )


def is_valid_get_discoveries_by_range(obj):
    json_schema_validate('jsd_33b799d04d0a8907_v1_2_10').validate(obj)
    return True


def get_discoveries_by_range(api):
    endpoint_result = api.network_discovery.get_discoveries_by_range(
        records_to_return=10,
        start_index=1,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_discoveries_by_range(api):
    assert is_valid_get_discoveries_by_range(
        get_discoveries_by_range(api)
    )


def is_valid_get_network_devices_from_discovery(obj):
    json_schema_validate('jsd_3d9b99c343398a27_v1_2_10').validate(obj)
    return True


def get_network_devices_from_discovery(api):
    endpoint_result = api.network_discovery.get_network_devices_from_discovery(
        id=get_discoveries_by_range(api).response[0].id,
        cli_status=None,
        http_status=None,
        ip_address=None,
        netconf_status=None,
        ping_status=None,
        snmp_status=None,
        sort_by=None,
        sort_order=None,
        task_id=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_network_devices_from_discovery(api):
    assert is_valid_get_network_devices_from_discovery(
        get_network_devices_from_discovery(api)
    )


def is_valid_get_discovery_by_id(obj):
    json_schema_validate('jsd_63bb88b74f59aa17_v1_2_10').validate(obj)
    return True


def get_discovery_by_id(api):
    endpoint_result = api.network_discovery.get_discovery_by_id(
        id=get_discoveries_by_range(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_discovery_by_id(api):
    assert is_valid_get_discovery_by_id(
        get_discovery_by_id(api)
    )


def is_valid_get_list_of_discoveries_by_discovery_id(obj):
    json_schema_validate('jsd_99872a134d0a9fb4_v1_2_10').validate(obj)
    return True


def get_list_of_discoveries_by_discovery_id(api):
    endpoint_result = api.network_discovery.get_list_of_discoveries_by_discovery_id(
        id=get_discoveries_by_range(api).response[0].id,
        ip_address=None,
        limit=None,
        offset=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_list_of_discoveries_by_discovery_id(api):
    assert is_valid_get_list_of_discoveries_by_discovery_id(
        get_list_of_discoveries_by_discovery_id(api)
    )


def is_valid_get_discovery_jobs_by_ip(obj):
    json_schema_validate('jsd_a4967be64dfaaa1a_v1_2_10').validate(obj)
    return True


def get_discovery_jobs_by_ip(api):
    endpoint_result = api.network_discovery.get_discovery_jobs_by_ip(
        ip_address=get_discoveries_by_range(api).response[0].ipAddressList.split('-')[0],
        limit=None,
        name=None,
        offset=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_discovery_jobs_by_ip(api):
    assert is_valid_get_discovery_jobs_by_ip(
        get_discovery_jobs_by_ip(api)
    )


def is_valid_get_discovered_devices_by_range(obj):
    json_schema_validate('jsd_a6b798ab4acaa34e_v1_2_10').validate(obj)
    return True


def get_discovered_devices_by_range(api):
    discoveries = get_discoveries_by_range(api).response
    filtered_discoveries = discoveries
    # list(filter(lambda x: x.numDevices and x.numDevices > 0, discoveries))
    endpoint_result = api.network_discovery.get_discovered_devices_by_range(
        id=filtered_discoveries[0].id,
        records_to_return=3,
        start_index=1,
        task_id=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_discovered_devices_by_range(api):
    assert is_valid_get_discovered_devices_by_range(
        get_discovered_devices_by_range(api)
    )


def is_valid_get_devices_discovered_by_id(obj):
    json_schema_validate('jsd_a6965b454c9a8663_v1_2_10').validate(obj)
    return True


def get_devices_discovered_by_id(api):
    endpoint_result = api.network_discovery.get_devices_discovered_by_id(
        id=get_discoveries_by_range(api).response[0].id,
        task_id=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_devices_discovered_by_id(api):
    assert is_valid_get_devices_discovered_by_id(
        get_devices_discovered_by_id(api)
    )


def is_valid_get_global_credentials(obj):
    json_schema_validate('jsd_ff816b8e435897eb_v1_2_10').validate(obj)
    return True


def get_global_credentials(api):
    endpoint_result = api.network_discovery.get_global_credentials(
        credential_sub_type='CLI',
        order=None,
        sort_by=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_global_credentials(api):
    assert is_valid_get_global_credentials(
        get_global_credentials(api)
    )


def is_valid_get_discovered_network_devices_by_discovery_id(obj):
    json_schema_validate('jsd_f6ac994f451ba011_v1_2_10').validate(obj)
    return True


def get_discovered_network_devices_by_discovery_id(api):
    endpoint_result = api.network_discovery.get_discovered_network_devices_by_discovery_id(
        id=get_discoveries_by_range(api).response[0].id,
        task_id=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_get_discovered_network_devices_by_discovery_id(api):
    assert is_valid_get_discovered_network_devices_by_discovery_id(
        get_discovered_network_devices_by_discovery_id(api)
    )


def is_valid_delete_discovery_by_specified_range(obj):
    json_schema_validate('jsd_c1ba9a424c08a01b_v1_2_10').validate(obj)
    return True


def delete_discovery_by_specified_range(api):
    endpoint_result = api.network_discovery.delete_discovery_by_specified_range(
        records_to_delete=3,
        start_index=800,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_discovery_by_specified_range(api):
    assert is_valid_delete_discovery_by_specified_range(
        delete_discovery_by_specified_range(api)
    )


def is_valid_update_netconf_credentials(obj):
    json_schema_validate('jsd_c5acd9fa4c1a8abc_v1_2_10').validate(obj)
    return True


def update_netconf_credentials(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="NETCONF").response
    endpoint_result = api.network_discovery.update_netconf_credentials(
        comments=None,
        credentialType=None,
        description=None,
        id=list(filter(lambda x: x.netconfPort == '65533', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        netconfPort='65532',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_netconf_credentials(api):
    assert is_valid_update_netconf_credentials(
        update_netconf_credentials(api)
    )


def is_valid_update_global_credentials(obj):
    json_schema_validate('jsd_709fda3c42b8877a_v1_2_10').validate(obj)
    return True


def update_global_credentials(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="NETCONF").response
    endpoint_result = api.network_discovery.update_global_credentials(
        global_credential_id=list(filter(lambda x: x.netconfPort == '65532', credentials))[0].id,
        siteUuids=[],
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_global_credentials(api):
    assert is_valid_update_global_credentials(
        update_global_credentials(api)
    )


def is_valid_delete_global_credentials_by_id_netconf(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_netconf(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="NETCONF").response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.netconfPort == '65532', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_netconf(api):
    assert is_valid_delete_global_credentials_by_id_netconf(
        delete_global_credentials_by_id_netconf(api)
    )


def is_valid_update_snmp_write_community(obj):
    json_schema_validate('jsd_10b06a6a4f7bb3cb_v1_2_10').validate(obj)
    return True


def update_snmp_write_community(api):
    time.sleep(10)
    sub_type = "SNMPV2_WRITE_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    endpoint_result = api.network_discovery.update_snmp_write_community(
        comments=None,
        credentialType=None,
        description='created snmpv2_write',
        id=list(filter(lambda x: x.description == 'created snmpv2', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        writeCommunity='NO!$DATA!$',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_snmp_write_community(api):
    assert is_valid_update_snmp_write_community(
        update_snmp_write_community(api)
    )


def is_valid_update_snmp_read_community(obj):
    json_schema_validate('jsd_47a1b84b4e1b8044_v1_2_10').validate(obj)
    return True


def update_snmp_read_community(api):
    time.sleep(10)
    sub_type = "SNMPV2_READ_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    endpoint_result = api.network_discovery.update_snmp_read_community(
        comments=None,
        credentialType=None,
        description='created snmpv2_read',
        id=list(filter(lambda x: x.description == 'created snmpv2', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        readCommunity='NO!$DATA!$',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_snmp_read_community(api):
    assert is_valid_update_snmp_read_community(
        update_snmp_read_community(api)
    )


def is_valid_update_cli_credentials(obj):
    json_schema_validate('jsd_fba0d80747eb82e8_v1_2_10').validate(obj)
    return True


def update_cli_credentials(api):
    time.sleep(10)
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="CLI").response
    endpoint_result = api.network_discovery.update_cli_credentials(
        comments=None,
        credentialType=None,
        description='test: user devnet credentials',
        enablePassword=None,
        id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        password='NO!$DATA!$',
        username='test_user_devnet',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_cli_credentials(api):
    assert is_valid_update_cli_credentials(
        update_cli_credentials(api)
    )


def is_valid_delete_global_credentials_by_id_snmp_write(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_snmp_write(api):
    time.sleep(10)
    sub_type = "SNMPV2_WRITE_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.description == 'created snmpv2_write', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_snmp_write(api):
    assert is_valid_delete_global_credentials_by_id_snmp_write(
        delete_global_credentials_by_id_snmp_write(api)
    )


def is_valid_delete_global_credentials_by_id_snmp_read(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_snmp_read(api):
    time.sleep(10)
    sub_type = "SNMPV2_READ_COMMUNITY"
    credentials = api.network_discovery.get_global_credentials(credential_sub_type=sub_type).response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.description == 'created snmpv2_read', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_snmp_read(api):
    assert is_valid_delete_global_credentials_by_id_snmp_read(
        delete_global_credentials_by_id_snmp_read(api)
    )


def is_valid_delete_global_credentials_by_id_cli(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_cli(api):
    time.sleep(10)
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="CLI").response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_cli(api):
    assert is_valid_delete_global_credentials_by_id_cli(
        delete_global_credentials_by_id_cli(api)
    )


def is_valid_update_http_write_credentials(obj):
    json_schema_validate('jsd_b68a6bd8473a9a25_v1_2_10').validate(obj)
    return True


def update_http_write_credentials(api):
    time.sleep(10)
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="HTTP_WRITE").response
    endpoint_result = api.network_discovery.update_http_write_credentials(
        comments=None,
        credentialType=None,
        description='created http_write',
        id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        password='W.~&KV9ha',
        port=8080,
        secure=None,
        username='test_user_devnet',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_http_write_credentials(api):
    assert is_valid_update_http_write_credentials(
        update_http_write_credentials(api)
    )


def is_valid_update_http_read_credential(obj):
    json_schema_validate('jsd_89b36b4649999d81_v1_2_10').validate(obj)
    return True


def update_http_read_credential(api):
    time.sleep(10)
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="HTTP_READ").response
    endpoint_result = api.network_discovery.update_http_read_credential(
        comments=None,
        credentialType=None,
        description='created http_write',
        id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        password='W.~&KV9ha',
        port=8080,
        secure=None,
        username='test_user_devnet',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_http_read_credential(api):
    assert is_valid_update_http_read_credential(
        update_http_read_credential(api)
    )


def is_valid_update_snmpv3_credentials(obj):
    json_schema_validate('jsd_1da5ebdd434aacfe_v1_2_10').validate(obj)
    return True


def update_snmpv3_credentials(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="SNMPV3").response
    endpoint_result = api.network_discovery.update_snmpv3_credentials(
        authPassword=None,
        authType=None,
        comments=None,
        credentialType=None,
        description='created snmpv3',
        id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        instanceTenantId=None,
        instanceUuid=None,
        privacyPassword=None,
        privacyType=None,
        snmpMode='NOAUTHNOPRIV',
        username='test_user_devnet',
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_update_snmpv3_credentials(api):
    assert is_valid_update_snmpv3_credentials(
        update_snmpv3_credentials(api)
    )


def is_valid_delete_global_credentials_by_id_http_write(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_http_write(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="HTTP_WRITE").response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_http_write(api):
    assert is_valid_delete_global_credentials_by_id_http_write(
        delete_global_credentials_by_id_http_write(api)
    )


def is_valid_delete_global_credentials_by_id_http_read(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_http_read(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="HTTP_READ").response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_http_read(api):
    assert is_valid_delete_global_credentials_by_id_http_read(
        delete_global_credentials_by_id_http_read(api)
    )


def is_valid_delete_global_credentials_by_id_snmpv3(obj):
    json_schema_validate('jsd_f5ac590c4ca9975a_v1_2_10').validate(obj)
    return True


def delete_global_credentials_by_id_snmpv3(api):
    credentials = api.network_discovery.get_global_credentials(credential_sub_type="SNMPV3").response
    endpoint_result = api.network_discovery.delete_global_credentials_by_id(
        global_credential_id=list(filter(lambda x: x.username == 'test_user_devnet', credentials))[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_global_credentials_by_id_snmpv3(api):
    assert is_valid_delete_global_credentials_by_id_snmpv3(
        delete_global_credentials_by_id_snmpv3(api)
    )


def is_valid_updates_discovery_by_id_active(obj):
    json_schema_validate('jsd_9788b8fc4418831d_v1_2_10').validate(obj)
    return True


def updates_discovery_by_id_active(api):
    discovery = get_discoveries_by_range(api).response[0]
    endpoint_result = api.network_discovery.updates_discovery_by_id(
        attributeInfo=None,
        cdpLevel=None,
        deviceIds=None,
        discoveryCondition=None,
        discoveryStatus=discovery.discoveryStatus,
        discoveryType=None,
        enablePasswordList=None,
        globalCredentialIdList=None,
        httpReadCredential=None,
        httpWriteCredential=None,
        id=discovery.id,
        ipAddressList=None,
        ipFilterList=None,
        isAutoCdp=None,
        lldpLevel=None,
        name=None,
        netconfPort=None,
        numDevices=None,
        parentDiscoveryId=None,
        passwordList=None,
        preferredMgmtIPMethod=None,
        protocolOrder=None,
        retryCount=None,
        snmpAuthPassphrase=None,
        snmpAuthProtocol=None,
        snmpMode=None,
        snmpPrivPassphrase=None,
        snmpPrivProtocol=None,
        snmpRoCommunity=None,
        snmpRoCommunityDesc=None,
        snmpRwCommunity=None,
        snmpRwCommunityDesc=None,
        snmpUserName=None,
        timeOut=None,
        updateMgmtIp=None,
        userNameList=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_updates_discovery_by_id_active(api):
    assert is_valid_updates_discovery_by_id_active(
        updates_discovery_by_id_active(api)
    )


def is_valid_updates_discovery_by_id_inactive(obj):
    json_schema_validate('jsd_9788b8fc4418831d_v1_2_10').validate(obj)
    return True


def updates_discovery_by_id_inactive(api):
    discovery = get_discoveries_by_range(api).response[0]
    endpoint_result = api.network_discovery.updates_discovery_by_id(
        attributeInfo=None,
        cdpLevel=None,
        deviceIds=None,
        discoveryCondition=None,
        discoveryStatus=discovery.discoveryStatus,
        discoveryType=None,
        enablePasswordList=None,
        globalCredentialIdList=None,
        httpReadCredential=None,
        httpWriteCredential=None,
        id=discovery.id,
        ipAddressList=None,
        ipFilterList=None,
        isAutoCdp=None,
        lldpLevel=None,
        name=None,
        netconfPort=None,
        numDevices=None,
        parentDiscoveryId=None,
        passwordList=None,
        preferredMgmtIPMethod=None,
        protocolOrder=None,
        retryCount=None,
        snmpAuthPassphrase=None,
        snmpAuthProtocol=None,
        snmpMode=None,
        snmpPrivPassphrase=None,
        snmpPrivProtocol=None,
        snmpRoCommunity=None,
        snmpRoCommunityDesc=None,
        snmpRwCommunity=None,
        snmpRwCommunityDesc=None,
        snmpUserName=None,
        timeOut=None,
        updateMgmtIp=None,
        userNameList=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_updates_discovery_by_id_inactive(api):
    assert is_valid_updates_discovery_by_id_inactive(
        updates_discovery_by_id_inactive(api)
    )


def is_valid_delete_discovery_by_id(obj):
    json_schema_validate('jsd_4c8cab5f435a80f4_v1_2_10').validate(obj)
    return True


def delete_discovery_by_id(api):
    time.sleep(10)
    discoveries = api.network_discovery.get_discoveries_by_range(
        records_to_return=500,
        start_index=1,
        payload=None
    ).response
    discovery_to_delete = list(filter(
        lambda x: x.ipAddressList == '10.10.22.22' and x.name == 'start_discovery_test', discoveries
    ))
    endpoint_result = api.network_discovery.delete_discovery_by_id(
        id=discovery_to_delete[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.network_discovery
def test_delete_discovery_by_id(api):
    assert is_valid_delete_discovery_by_id(
        delete_discovery_by_id(api)
    )



# -*- coding: utf-8 -*-
"""DNACenterAPI template_programmer API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_gets_the_templates_available(obj):
    json_schema_validate('jsd_01b09a254b9ab259_v1_2_10').validate(obj)
    return True


def gets_the_templates_available(api):
    endpoint_result = api.template_programmer.gets_the_templates_available(
        filter_conflicting_templates=None,
        product_family=None,
        product_series=None,
        product_type=None,
        project_id=None,
        software_type=None,
        software_version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_gets_the_templates_available(api):
    assert is_valid_gets_the_templates_available(
        gets_the_templates_available(api)
    )


def is_valid_get_projects(obj):
    json_schema_validate('jsd_109d1b4f4289aecd_v1_2_10').validate(obj)
    return True


def get_projects(api):
    endpoint_result = api.template_programmer.get_projects(
        name=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_get_projects(api):
    assert is_valid_get_projects(
        get_projects(api)
    )


def is_valid_get_template_details(obj):
    json_schema_validate('jsd_83a3b9404cb88787_v1_2_10').validate(obj)
    return True


def get_template_details(api):
    templates_available = gets_the_templates_available(api)
    projects = get_projects(api)
    if len(projects) > 0 and len(projects[0].templates) > 0:
        templateID = projects[0].templates[0].id
    if len(templates_available) > 0:
        templateID = templates_available[0].templateId
    endpoint_result = api.template_programmer.get_template_details(
        template_id=templateID,
        latest_version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_get_template_details(api):
    assert is_valid_get_template_details(
        get_template_details(api)
    )


def is_valid_get_template_versions(obj):
    json_schema_validate('jsd_c8bf6b65414a9bc7_v1_2_10').validate(obj)
    return True


def get_template_versions(api):
    templates_available = gets_the_templates_available(api)
    projects = get_projects(api)
    if len(projects) > 0 and len(projects[0].templates) > 0:
        templateID = projects[0].templates[0].id
    if len(templates_available) > 0:
        templateID = templates_available[0].templateId
    endpoint_result = api.template_programmer.get_template_versions(
        template_id=templateID,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_get_template_versions(api):
    assert is_valid_get_template_versions(
        get_template_versions(api)
    )


def is_valid_create_project(obj):
    json_schema_validate('jsd_00aec9b1422ab27e_v1_2_10').validate(obj)
    return True


def create_project(api):
    endpoint_result = api.template_programmer.create_project(
        createTime=None,
        description=None,
        id=None,
        lastUpdateTime=None,
        name='test_project',
        tags=None,
        templates=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_create_project(api):
    assert is_valid_create_project(
        create_project(api)
    )


def is_valid_create_template(obj):
    json_schema_validate('jsd_f6b119ad4d4aaf16_v1_2_10').validate(obj)
    return True


def create_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project', projects))
    endpoint_result = api.template_programmer.create_template(
        project_id=filtered_project[0].id,
        author=None,
        composite=False,
        containingTemplates=[],
        createTime=None,
        description=None,
        deviceTypes=[
            {
                "productFamily": "Switches and Hubs",
                # "productSeries": "Cisco Catalyst 9300 Series Switches",
                # "productType": "Cisco Catalyst 9300 Switch"
            }
        ],
        failurePolicy=None,
        id=None,
        lastUpdateTime=None,
        name='test_template',
        parentTemplateId=None,
        projectId=None,
        projectName=None,
        rollbackTemplateContent='',
        rollbackTemplateParams=[],
        softwareType='IOS-XE',
        softwareVariant='XE',
        softwareVersion=None,
        tags=None,
        templateContent='show version\n',
        templateParams=[],
        version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_create_template(api):
    assert is_valid_create_template(
        create_template(api)
    )


def is_valid_update_template(obj):
    json_schema_validate('jsd_7781fa0548a98342_v1_2_10').validate(obj)
    return True


def update_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project', projects))
    template = api.template_programmer.get_template_details(template_id=filtered_project[0].templates[0].id)
    endpoint_result = api.template_programmer.update_template(
        author=None,
        composite=template.composite,
        containingTemplates=template.containingTemplates,
        createTime=None,
        description=None,
        deviceTypes=template.deviceTypes,
        failurePolicy=None,
        id=template.id,
        lastUpdateTime=None,
        name=template.name + '_updated',
        parentTemplateId=None,
        projectId=template.projectId,
        projectName=None,
        rollbackTemplateContent=template.rollbackTemplateContent,
        rollbackTemplateParams=template.rollbackTemplateParams,
        softwareType=template.softwareType,
        softwareVariant=template.softwareVariant,
        softwareVersion=None,
        tags=None,
        templateContent=template.templateContent,
        templateParams=template.templateParams,
        version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_update_template(api):
    assert is_valid_update_template(
        update_template(api)
    )


def is_valid_update_project(obj):
    json_schema_validate('jsd_9480fa1f47ca9254_v1_2_10').validate(obj)
    return True


def update_project(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project', projects))
    endpoint_result = api.template_programmer.update_project(
        createTime=None,
        description=None,
        id=filtered_project[0].id,
        lastUpdateTime=None,
        name=filtered_project[0].name + '_updated',
        tags=None,
        templates=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_update_project(api):
    assert is_valid_update_project(
        update_project(api)
    )


def is_valid_version_template(obj):
    json_schema_validate('jsd_62b05b2c40a9b216_v1_2_10').validate(obj)
    return True


def version_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project_updated', projects))
    template = api.template_programmer.get_template_details(template_id=filtered_project[0].templates[0].id)
    endpoint_result = api.template_programmer.version_template(
        comments=None,
        templateId=template.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_version_template(api):
    assert is_valid_version_template(
        version_template(api)
    )


def is_valid_preview_template(obj):
    json_schema_validate('jsd_f393abe84989bb48_v1_2_10').validate(obj)
    return True


def preview_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project_updated', projects))
    template = api.template_programmer.get_template_details(template_id=filtered_project[0].templates[0].id)
    endpoint_result = api.template_programmer.preview_template(
        params=None,
        templateId=template.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_preview_template(api):
    assert is_valid_preview_template(
        preview_template(api)
    )


def is_valid_deploy_template(obj):
    json_schema_validate('jsd_6099da82477b858a_v1_2_10').validate(obj)
    return True


def deploy_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project_updated', projects))
    template = api.template_programmer.get_template_details(template_id=filtered_project[0].templates[0].id)
    target = api.devices.get_device_list(family=template.deviceTypes[0].productFamily,
                                         software_type=template.softwareType).response
    # versions = api.template_programmer.get_template_versions(template.id)[0].versionsInfo
    # versions = sorted(versions, key=lambda version: str(version.get('version', -1)), reverse=True)
    endpoint_result = api.template_programmer.deploy_template(
        forcePushTemplate=True,
        isComposite=template.composite,
        mainTemplateId=template.parentTemplateId,
        memberTemplateDeploymentInfo=None,
        targetInfo=[{'id': t.managementIpAddress,
                    'type': 'MANAGED_DEVICE_IP', 'params': {}} for t in target],
        templateId=template.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_deploy_template(api):
    assert is_valid_deploy_template(
        deploy_template(api)
    )


def is_valid_get_template_deployment_status(obj):
    json_schema_validate('jsd_9c9a785741cbb41f_v1_2_10').validate(obj)
    return True


def get_template_deployment_status(api):
    time.sleep(10)
    deploymentID = deploy_template(api).deploymentId.split(" ")[-1]
    endpoint_result = api.template_programmer.get_template_deployment_status(
        deployment_id=deploymentID,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_get_template_deployment_status(api):
    assert is_valid_get_template_deployment_status(
        get_template_deployment_status(api)
    )


def is_valid_delete_template(obj):
    json_schema_validate('jsd_a7b42836408a8e74_v1_2_10').validate(obj)
    return True


def delete_template(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project_updated', projects))
    template = api.template_programmer.get_template_details(template_id=filtered_project[0].templates[0].id)
    endpoint_result = api.template_programmer.delete_template(
        template_id=template.id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_delete_template(api):
    assert is_valid_delete_template(
        delete_template(api)
    )


def is_valid_delete_project(obj):
    json_schema_validate('jsd_d0a1abfa435b841d_v1_2_10').validate(obj)
    return True


def delete_project(api):
    projects = api.template_programmer.get_projects()
    filtered_project = list(filter(lambda x: x.name == 'test_project_updated', projects))
    endpoint_result = api.template_programmer.delete_project(
        project_id=filtered_project[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.template_programmer
def test_delete_project(api):
    assert is_valid_delete_project(
        delete_project(api)
    )



# -*- coding: utf-8 -*-
"""DNACenterAPI path_trace API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_initiate_a_new_pathtrace(obj):
    json_schema_validate('jsd_a395fae644ca899c_v1_2_10').validate(obj)
    return True


def initiate_a_new_pathtrace(api):
    discoveries = api.network_discovery.get_discoveries_by_range(records_to_return=10, start_index=1).response
    ipAddressLists = [list(set(discovery.ipAddressList.split('-'))) if discovery.ipAddressList else set()
                      for discovery in discoveries]
    ipAddressList = list(set([ip for iplist in ipAddressLists for ip in iplist]))
    ipAddressList.sort()
    endpoint_result = api.path_trace.initiate_a_new_pathtrace(
        controlPath=None,
        destIP=ipAddressList[-1] if len(ipAddressList) > 0 else "10.20.10.1",
        destPort=None,
        inclusions=None,
        periodicRefresh=None,
        protocol=None,
        sourceIP=ipAddressList[0] if len(ipAddressList) > 0 else "10.20.10.1",
        sourcePort=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.path_trace
def test_initiate_a_new_pathtrace(api):
    assert is_valid_initiate_a_new_pathtrace(
        initiate_a_new_pathtrace(api)
    )


def is_valid_deletes_pathtrace_by_id(obj):
    json_schema_validate('jsd_8a9d2b76443b914e_v1_2_10').validate(obj)
    return True


def deletes_pathtrace_by_id(api):
    endpoint_result = api.path_trace.deletes_pathtrace_by_id(
        flow_analysis_id=retrives_all_previous_pathtraces_summary(api).response[0].id,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.path_trace
def test_deletes_pathtrace_by_id(api):
    assert is_valid_deletes_pathtrace_by_id(
        deletes_pathtrace_by_id(api)
    )


def is_valid_retrieves_previous_pathtrace(obj):
    json_schema_validate('jsd_7ab9a8bd4f3b86a4_v1_2_10').validate(obj)
    return True


def retrieves_previous_pathtrace(api):
    endpoint_result = api.path_trace.retrieves_previous_pathtrace(
        flow_analysis_id=initiate_a_new_pathtrace(api).response.flowAnalysisId,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.path_trace
def test_retrieves_previous_pathtrace(api):
    assert is_valid_retrieves_previous_pathtrace(
        retrieves_previous_pathtrace(api)
    )


def is_valid_retrives_all_previous_pathtraces_summary(obj):
    json_schema_validate('jsd_55bc3bf94e38b6ff_v1_2_10').validate(obj)
    return True


def retrives_all_previous_pathtraces_summary(api):
    discoveries = api.network_discovery.get_discoveries_by_range(records_to_return=10, start_index=1).response
    ipAddressLists = [list(set(discovery.ipAddressList.split('-'))) if discovery.ipAddressList else set()
                      for discovery in discoveries]
    ipAddressList = list(set([ip for iplist in ipAddressLists for ip in iplist]))
    ipAddressList.sort()
    endpoint_result = api.path_trace.retrives_all_previous_pathtraces_summary(
        dest_ip=None,
        dest_port=None,
        gt_create_time=None,
        last_update_time=None,
        limit=None,
        lt_create_time=None,
        offset=None,
        order=None,
        periodic_refresh=None,
        protocol=None,
        sort_by=None,
        source_ip=ipAddressList[0] if len(ipAddressList) > 0 else "10.20.10.1",
        source_port=None,
        status=None,
        task_id=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.path_trace
def test_retrives_all_previous_pathtraces_summary(api):
    assert is_valid_retrives_all_previous_pathtraces_summary(
        retrives_all_previous_pathtraces_summary(api)
    )



# -*- coding: utf-8 -*-
"""DNACenterAPI swim API fixtures and tests.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import pytest
import dnacentersdk
import time
from tests.environment import DNA_CENTER_VERSION
from tests.models.schema_validator import json_schema_validate
from tests.config import LOCAL_SOFTWARE_IMAGE_PATH, LOCAL_SOFTWARE_IMAGE_NAME
from tests.config import DEFAULT_BASE_URL
from .test_file import get_list_of_files
from .test_devices import get_device_list


pytestmark = pytest.mark.skipif(DNA_CENTER_VERSION != '1.2.10', reason='version does not match')


def is_valid_get_software_image_details(obj):
    json_schema_validate('jsd_0c8f7a0b49b9aedd_v1_2_10').validate(obj)
    return True


def get_software_image_details(api):
    endpoint_result = api.swim.get_software_image_details(
        application_type=None,
        created_time=None,
        family=None,
        image_integrity_status=None,
        image_name=None,
        image_series=None,
        image_size_greater_than=None,
        image_size_lesser_than=None,
        image_uuid=None,
        is_cco_latest=None,
        is_cco_recommended=None,
        is_tagged_golden=None,
        limit=None,
        name=None,
        offset=None,
        sort_by=None,
        sort_order='asc',
        version=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.swim
def test_get_software_image_details(api):
    assert is_valid_get_software_image_details(
        get_software_image_details(api)
    )


def is_valid_import_local_software_image(obj):
    json_schema_validate('jsd_4dbe3bc743a891bc_v1_2_10').validate(obj)
    return True


def import_local_software_image(api):
    endpoint_result = api.swim.import_local_software_image(
        is_third_party=None,
        third_party_application_type=None,
        third_party_image_family=None,
        third_party_vendor=None,
        multipart_fields={'file': (LOCAL_SOFTWARE_IMAGE_NAME,
                                   open(LOCAL_SOFTWARE_IMAGE_PATH, 'rb'))},
        multipart_monitor_callback=None,
        payload=None,
        active_validation=True
    )
    return endpoint_result


@pytest.mark.skipif(not all([LOCAL_SOFTWARE_IMAGE_PATH, LOCAL_SOFTWARE_IMAGE_PATH]) is True,
                    reason="tests.config values required not present")
@pytest.mark.swim
def test_import_local_software_image(api):
    assert is_valid_import_local_software_image(
        import_local_software_image(api)
    )


def is_valid_import_software_image_via_url(obj):
    json_schema_validate('jsd_bc8aab4746ca883d_v1_2_10').validate(obj)
    return True


def import_software_image_via_url(api):
    image_details = get_software_image_details(api).response[0]
    files = get_list_of_files(api).response[0].downloadPath
    endpoint_result = api.swim.import_software_image_via_url(
        schedule_at=None,
        schedule_desc=None,
        schedule_origin=None,
        payload=[{
            'applicationType': image_details.applicationType,
            'vendor': image_details.vendor,
            'imageFamily': image_details.family,
            'sourceURL': DEFAULT_BASE_URL + '/dna/intent/api/v1/' + files
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.skipif(not all([DEFAULT_BASE_URL]) is True,
                    reason="tests.config values required not present")
@pytest.mark.swim
def test_import_software_image_via_url(api):
    assert is_valid_import_software_image_via_url(
        import_software_image_via_url(api)
    )


def is_valid_trigger_software_image_activation(obj):
    json_schema_validate('jsd_fb9beb664f2aba4c_v1_2_10').validate(obj)
    return True


def trigger_software_image_activation(api):
    endpoint_result = api.swim.trigger_software_image_activation(
        schedule_validate=None,
        payload=[{
            'activateLowerImageVersion': True,
            'deviceUuid': get_device_list(api).response[0].id,
            'distributeIfNeeded': True,
            'imageUuidList': [get_software_image_details(api).response[0].imageUuid],
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.swim
def test_trigger_software_image_activation(api):
    assert is_valid_trigger_software_image_activation(
        trigger_software_image_activation(api)
    )


def is_valid_trigger_software_image_distribution(obj):
    json_schema_validate('jsd_8cb6783b4faba1f4_v1_2_10').validate(obj)
    return True


def trigger_software_image_distribution(api):
    endpoint_result = api.swim.trigger_software_image_distribution(
        payload=[{
            'deviceUuid': get_device_list(api).response[0].id,
            'imageUuid': get_software_image_details(api).response[0].imageUuid
        }],
        active_validation=True
    )
    return endpoint_result


@pytest.mark.swim
def test_trigger_software_image_distribution(api):
    assert is_valid_trigger_software_image_distribution(
        trigger_software_image_distribution(api)
    )
